@@ -1,24 +1,35 @@
 from Products.CMFBibliographyAT.config import CMFBAT_USES_LINGUAPLONE
 if CMFBAT_USES_LINGUAPLONE:
-    from Products.LinguaPlone.public import BaseSchema, Schema
-    from Products.LinguaPlone.public import StringField, TextField, LinesField, ReferenceField, FileField
-    from Products.LinguaPlone.public import SelectionWidget, ReferenceWidget, TextAreaWidget, FileWidget
-    from Products.LinguaPlone.public import RichWidget, IdWidget, StringWidget
-    from Products.LinguaPlone.public import DisplayList
+    from Products.LinguaPlone.public import Schema
+    from Products.LinguaPlone.public import StringField, TextField, LinesField
+    from Products.LinguaPlone.public import ReferenceField, FileField
+    from Products.LinguaPlone.public import ReferenceWidget
+    from Products.LinguaPlone.public import TextAreaWidget
+    from Products.LinguaPlone.public import FileWidget
+    from Products.LinguaPlone.public import RichWidget, StringWidget
+    from Products.LinguaPlone.public import KeywordWidget
 else:
-    from Products.Archetypes.public import BaseSchema, Schema
-    from Products.Archetypes.public import StringField, TextField, LinesField, ReferenceField, FileField
-    from Products.Archetypes.public import SelectionWidget, ReferenceWidget, TextAreaWidget, FileWidget
-    from Products.Archetypes.public import RichWidget, IdWidget, StringWidget
-    from Products.Archetypes.public import DisplayList
+    from Products.Archetypes.public import Schema
+    from Products.Archetypes.public import StringField, TextField, LinesField
+    from Products.Archetypes.public import ReferenceField, FileField
+    from Products.Archetypes.public import ReferenceWidget
+    from Products.Archetypes.public import TextAreaWidget
+    from Products.Archetypes.public import FileWidget
+    from Products.Archetypes.public import RichWidget, StringWidget
+    from Products.Archetypes.public import KeywordWidget
+
+from Products.Archetypes import PloneMessageFactory as _
+from Products.ATContentTypes.content.schemata \
+     import ATContentTypeSchema as BaseSchema
 
-from Products.Archetypes.Widget import TypesWidget
 from Products.ATReferenceBrowserWidget.ATReferenceBrowserWidget \
      import ReferenceBrowserWidget
 
 from Products.CMFBibliographyAT.marshall import BibtexMarshaller
-from Products.ATExtensions.ateapi import FormattableNamesField, FormattableNamesWidget, \
-                                         CommentField, CommentWidget
+from Products.ATExtensions.ateapi import FormattableNamesField
+from Products.ATExtensions.ateapi import FormattableNamesWidget
+from Products.ATExtensions.ateapi import CommentField, CommentWidget
+from Products.ATExtensions.ateapi import RecordsField, RecordsWidget
 
 
 HeaderSchema = BaseSchema.copy()
@@ -26,21 +37,21 @@ HeaderSchema['id'].widget.macro_edit = "widgets/string"
 # XXX ugly hack as long as IdWidget subclasses TypesWidget
 HeaderSchema['id'].widget.size = 40
 HeaderSchema['id'].widget.maxlength = 255
-HeaderSchema['id'].widget.condition = 'python: not object.getBibFolder().getCookIdsAfterBibRefEdit()'
+HeaderSchema['id'].widget.condition = 'python: object.visibleIdsEnabled() and not object.getBibFolder().getCookIdsAfterBibRefEdit()'
 
 CookIdWarningField = StringField('shortname_cookid_warning',
                  schemata="default",
-		 mode="rw",
-		 accessor="getCookIdWarning",
-		 widget = StringWidget(
-		    label="Short Name",
-		    label_msgid="label_shortname_cookid_warning",
-		    description="There is ID re-cooking (after every edit / paste action) enabled on this item's parent bibliography folder. You cannot manually edit this item's short name. Modifications in this field will be ignored.",
-		    description_msgid="help_shortname_cookid_warning",
-		    i18n_domain="cmfbibliographyat",
-		    modes=('view', 'edit'),
-		    condition = 'python: object.getBibFolder().getCookIdsAfterBibRefEdit() and object.portal_membership.getAuthenticatedMember().getProperty("visible_ids", object.portal_memberdata.getProperty("visible_ids"))',
-		 ),
+                 mode="rw",
+                 accessor="getCookIdWarning",
+                 widget = StringWidget(
+                    label="Short Name",
+                    label_msgid="label_shortname_cookid_warning",
+                    description="There is ID re-cooking (after every edit / paste action) enabled on this item's parent bibliography folder. You cannot manually edit this item's short name. Modifications in this field will be ignored.",
+                    description_msgid="help_shortname_cookid_warning",
+                    i18n_domain="cmfbibliographyat",
+                    modes=('view', 'edit'),
+                    condition = 'python: object.getBibFolder().getCookIdsAfterBibRefEdit() and object.portal_membership.getAuthenticatedMember().getProperty("visible_ids", object.portal_memberdata.getProperty("visible_ids"))',
+                 ),
 )
 
 HeaderSchema.addField(CookIdWarningField)
@@ -50,30 +61,21 @@ tmp_title.is_duplicates_criterion=True
 del HeaderSchema['title']
 
 AuthorSchema = Schema((
-    ReferenceField('member_publication_authors',
-        relationship='authorOf',
-        mutator='setMemberPublicationAuthors',
-        languageIndependent=1,
-        multiValued=1,
-        widget=ReferenceWidget(label='Authors',
-            description='',
-            visible={'view':'invisible', 'edit':'invisible'},
-            condition='python:object.showMemberAuthors()',
-            i18n_domain="cmfbibliographyat",
-            ),
-        ),
     FormattableNamesField('authors', # was 'publication_authors',
         searchable = 1,
         required = 0,
         minimalSize = 2,
-        subfields=('reference','firstnames','lastname', 'homepage'),
+        subfields=('username','firstnames','lastname', 'homepage'),
         subfield_sizes={'firstnames':20, 'lastname':20, 'homepage':15},
-        subfield_vocabularies={'reference':'getSiteMembers'},
-        subfield_conditions={'reference':'python:object.showMemberAuthors()'},
+        subfield_labels={'username':'Site Members'},
+        subfield_vocabularies={'username':'getSiteMembers'},
+        subfield_conditions={'username':'python:object.showMemberAuthors()'},
         subfield_maxlength={'homepage': 250,},
-	is_duplicates_criterion=True,
+        is_duplicates_criterion=True,
         widget=FormattableNamesWidget(label="Authors",
             label_msgid="label_authors",
+            macro_edit = "authors_widget",
+            helper_js = ('authors_widget.js',),	
             description="If possible, always fill in the complete authors' / editors' names.",
             description_msgid="help_authors",
             i18n_domain="cmfbibliographyat",
@@ -87,7 +89,7 @@ CoreSchema = Schema((
         searchable=1,
         required=1,
         languageIndependent=1,
-	is_duplicates_criterion=True,
+        is_duplicates_criterion=True,
         widget=StringWidget(
             label="Publication Year",
             label_msgid="label_publication_year",
@@ -131,10 +133,10 @@ CoreSchema = Schema((
                    - Upload a printable (pdf) file""",
                  comment_msgid="comment_explain_links",
                  comment_type = "text/structured",
-        widget = CommentWidget(
+                 widget = CommentWidget(
                     i18n_domain="cmfbibliographyat",
                  ),
-    ), 
+    ),
     StringField('publication_url',
         schemata="full text",
         required=0,
@@ -147,6 +149,7 @@ CoreSchema = Schema((
             description="The (external) URL to get to an online version of the referenced resource.",
             description_msgid="help_url",
             i18n_domain="cmfbibliographyat",
+            size=60,
             ),
         ),
     StringField('pdf_url',
@@ -161,6 +164,7 @@ CoreSchema = Schema((
             description="The (external) URL to retrieve a printable version (PDF file) of the referenced resource from.",
             description_msgid="help_pdfref_url",
             i18n_domain="cmfbibliographyat",
+            size=60,
             ),
         ),
     ReferenceField('pdf_file',
@@ -176,9 +180,9 @@ CoreSchema = Schema((
                         description="This is AT field is hidden to anyone but portal managers. It refers to the associated PDF document on this site (if any). Use it for repair if the PDF file association of this bibliographical entry is broken.",
                         description_msgid="help_pdf_file",
                         addable=0,
-                        force_close_on_insert=True, 
+                        force_close_on_insert=True,
                         destination='getPdfFolderPath',
-                        startup_directory='getPdfFolderPath',
+                        startup_directory_method='getPdfFolderPath',
                         i18n_domain = "cmfbibliographyat",
                         visible={'edit': 'visible', 'view': 'invisible',},
                         condition="python: object.portal_membership.checkPermission('ManagePortal', object)",
@@ -190,7 +194,7 @@ CoreSchema = Schema((
                     default_content_type = "application/pdf",
                     mutator='setUploaded_pdfFile',
                     edit_accessor='editUploaded_pdfFile',
-                    accessor='getUploaded_pdfFile', 
+                    accessor='getUploaded_pdfFile',
                     #validators = (('isNonEmptyFile', V_REQUIRED),),
                     widget = FileWidget(
                                 label= "Printable PDF File",
@@ -202,21 +206,6 @@ CoreSchema = Schema((
                                 condition="python:object.portal_bibliography.allowPdfUploadPortalPolicy() and object.isPdfUploadAllowedForThisType()",
                     ),
     ),
-
-    StringField('supplementary_url',
-        schemata="full text",
-        required=0,
-        searchable=0,
-        validators=('isURL',),
-        widget=StringWidget(
-            label="Supplementary URL",
-            label_msgid="label_supplementary_url",
-            description="An (external) URL for referencing supplementary resources.",
-            description_msgid="help_url",
-            i18n_domain="cmfbibliographyat",
-            ),
-        ),
-
     ReferenceField('is_duplicate_of',
                     relationship="hasDuplicates",
                     multiValued=1,
@@ -229,11 +218,37 @@ CoreSchema = Schema((
 ))
 
 TrailingSchema = Schema((
+    RecordsField('identifiers',
+        searchable=0,
+        required=0,
+        languageIndependent=1,
+        is_duplicates_criterion=False,
+        subfields = ('label', 'value'),
+        subfield_labels ={'label':'Identifier'},
+        subfield_vocabularies = {'label':'publicationIdentifiers',},      
+        widget=RecordsWidget(
+                i18n_domain = "cmfbibliographyat",
+                label = u"Identifiers",
+                ),
+        ),
+    LinesField('keywords',
+        searchable=1,
+        required=0,
+        languageIndependent=1,
+        is_duplicates_criterion=False,
+        multiValued=1,
+        widget=KeywordWidget(
+                label=_(u'label_keywords', default=u'Keywords'),
+                description=_(u'help_keywords',
+                              default=u'Categorization of the publications content.'),
+                i18n_domain = "cmfbibliographyat",
+                ),
+        ),
     StringField('publication_month',
         searchable=1,
         required=0,
         languageIndependent=1,
-	is_duplicates_criterion=True,
+        is_duplicates_criterion=True,
         widget=StringWidget(
             label="Publication Month",
             label_msgid="label_publication_month",
@@ -245,7 +260,7 @@ TrailingSchema = Schema((
     TextField('note',
         searchable=1,
         required=0,
-	is_duplicates_criterion=True,
+        is_duplicates_criterion=True,
         widget=TextAreaWidget(label="Note",
             label_msgid="label_note",
             description_msgid="help_note",
@@ -256,7 +271,7 @@ TrailingSchema = Schema((
     TextField('annote',
         searchable=1,
         required=0,
-	is_duplicates_criterion=True,
+        is_duplicates_criterion=True,
         widget=TextAreaWidget(label="Annote",
             label_msgid="label_annote",
             description_msgid="help_annote",
@@ -264,5 +279,20 @@ TrailingSchema = Schema((
             i18n_domain="cmfbibliographyat",
             ),
         ),
+    RecordsField('additional',
+        searchable=0,
+        required=0,
+        languageIndependent=0,
+        is_duplicates_criterion=False,
+        subfields = ('key', 'value'),
+        subfield_vocabularies = {'key':'additionalFields',},  
+        innerJoin = ': ',
+        outerJoin = '<br />',
+        widget=RecordsWidget(
+                i18n_domain = "cmfbibliographyat",
+                label = u"Additional Fields",
+                condition="object/allowAdditionalFields",
+                ),
+        ),
     ), marshall = BibtexMarshaller())
 
