@@ -6,17 +6,18 @@
 ##########################################################################
 
 """BibliographyTool main class"""
-import os.path
-
 # Python stuff
-import re, types, string
-
+import re
+import string
+import codecs
+import logging
 
 # Zope stuff
-from Globals import InitializeClass, MessageDialog
+from zope.interface import implements
+from zope import component
+from Globals import InitializeClass
 from AccessControl import ClassSecurityInfo, ModuleSecurityInfo
 from OFS.Folder import Folder
-from Products.PageTemplates.PageTemplateFile import PageTemplateFile
 from persistent.mapping import PersistentMapping
 
 try:
@@ -26,28 +27,33 @@ except ImportError:
 
 # CMF stuff
 from Products.CMFCore.permissions import View, ManagePortal
-from Products.CMFCore.utils import getToolByName
 from Products.CMFCore.utils import UniqueObject
+from Products.CMFCore.utils import getToolByName
+from Products.Archetypes.atapi import DisplayList
 
 # My stuff ;-)
-from Products.CMFBibliographyAT.tool.parsers.base import ParserFolder
-from Products.CMFBibliographyAT.tool.renderers.base import RendererFolder
+from Products.CMFBibliographyAT.interface import IBibliographyTool
 from Products.CMFBibliographyAT.tool.idcookers.base import IdCookerFolder
-from Products.Archetypes.ArchetypeTool import ArchetypeTool
-from Products.CMFBibliographyAT.DuplicatesCriteria import DuplicatesCriteriaManager
-from Products.CMFBibliographyAT.config import REFERENCE_TYPES 
+from Products.CMFBibliographyAT.DuplicatesCriteria import \
+     DuplicatesCriteriaManager
+from Products.CMFBibliographyAT.config import REFERENCE_TYPES
 from Products.CMFBibliographyAT.config import FOLDER_TYPES as BIBFOLDER_TYPES
 from Products.CMFBibliographyAT.config import ZOPE_TEXTINDEXES
 from Products.CMFBibliographyAT import permissions
 
-from Products.CMFBibliographyAT.utils import _encode, _decode
+from bibliograph.core.utils import _encode
+from bibliograph.core.utils import _decode
+from bibliograph.rendering.interfaces import IBibliographyRenderer
+from bibliograph.parsing.interfaces import IBibliographyParser
+
 
-from bibutils import BibUtils
 
 # citation patterns
 citations = re.compile(r'\\?cite{([\w, ]*)}')
 bibitems = re.compile(r'\\?bibitem{([\w]*)}')
 
+LOG = logging.getLogger('CMFBibliographyAT')
+
 
 class ImportParseError(Exception):
     """An exception to replace the use of TypeError in
@@ -61,14 +67,18 @@ module_path = 'Products.CMFBibliographyAT.tool.bibliography'
 security = ModuleSecurityInfo(module_path)
 security.declarePublic('ImportParseError')
 
+from OFS.PropertySheets import PropertySheets
 
 class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
-                       DuplicatesCriteriaManager, BibUtils):
+                       DuplicatesCriteriaManager):
 
     """Tool for managing import and export functionality
        as well as some resources of the BibliographyFolders
        and -Entries.
     """
+
+    implements(IBibliographyTool)
+
     __allow_access_to_unprotected_subobjects__ = 1
 
     id = 'portal_bibliography'
@@ -95,7 +105,10 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
     allow_pdfupload_portal_policy = True
     allow_pdfupload_for_types = REFERENCE_TYPES
     searchable_bibfolders = True
-    
+    preprint_servers = []
+    allow_additional_fields = False
+    additional_fields = ['howpublished','recommendedby']  # just an example
+
     security = ClassSecurityInfo()
     security.declareObjectProtected(View)
 
@@ -108,7 +121,7 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
     _properties = Folder._properties + (
         {'id':'default_idcooker_id',
          'type':'selection',
-	 'select_variable':'listIdCookers', 
+         'select_variable':'listIdCookers',
          'mode':'w',
          },
         {'id':'cook_ids_on_bibref_creation',
@@ -204,13 +217,19 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
          'type':'boolean',
          'mode':'w',
          },
+        {'id':'allow_additional_fields',
+         'type':'boolean',
+         'mode':'w',
+         },
+        {'id':'additional_fields',
+         'type':'lines',
+         'mode':'w',
+         },
         )
     _match_criteria = {}
     _default_criteria = ('bibliography type',)
-    
+
     def __init__(self):
-        self._setObject('Parsers', ParserFolder('Parsers', ''))
-        self._setObject('Renderers', RendererFolder('Renderers', ''))
         self._setObject('IdCookers', IdCookerFolder('IdCookers', ''))
         DuplicatesCriteriaManager.__init__(self)
         # Add the local reference types registry
@@ -218,39 +237,39 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
         # Populate it initially with those types declared in config.py
         for ref_type in REFERENCE_TYPES:
             self._reference_types[ref_type] = None
-     
+
     def isBibFolderTranslatable(self):
         """ return what is in bibfolders_translatable property
             this is only half of the truth!!! (ITranslatable)
-        """    
-        return self.bibfolders_translatable 
+        """
+        return self.bibfolders_translatable
 
     def isBibrefItemTranslatable(self):
         """ return what is in bibrefitems_translatable property
             this is only half of the truth!!! (ITranslatable)
-        """    
+        """
         return self.bibrefitems_translatable
 
     security.declarePublic('getMemberTypes')
     def getMemberTypes(self, default=()):
-	member_types = self.getProperty('member_types', default)
-	if type(member_types) not in (types.ListType, types.TupleType,):
-	    return (member_types,)
-	else:
-	    return member_types
-        
+        member_types = self.getProperty('member_types', default)
+        if not isinstance(member_types, (list, tuple)):
+            return (member_types,)
+        else:
+            return member_types
+
     security.declarePublic('getSortMembersOn')
     def getSortMembersOn(self, default='getId'):
-	return self.getProperty('sort_members_on', default)
-        
+        return self.getProperty('sort_members_on', default)
+
     security.declarePublic('getSelectMembersAttr')
     def getSelectMembersAttr(self, default='Title'):
-	return self.getProperty('select_members_attr', default)
-        
+        return self.getProperty('select_members_attr', default)
+
     security.declarePublic('getMembersSearchOnAttr')
     def getMembersSearchOnAttr(self, default='Title'):
-	return self.getProperty('members_search_on_attr', default)
-        
+        return self.getProperty('members_search_on_attr', default)
+
     security.declarePublic('getReferenceTypes')
     def getReferenceTypes(self, display=False):
         """
@@ -290,15 +309,16 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
         try:
             del self._reference_types[portal_type]
         except KeyError:
-            pass 
+            pass
 
     security.declarePublic('getImportFormatNames')
     def getImportFormatNames(self, with_unavailables=False, with_disabled=False):
         """
         returns a list with the names of the supported import formats
         """
+        parsers = component.getAllUtilitiesRegisteredFor(IBibliographyParser)
         return [parser.getFormatName() \
-		for parser in self.Parsers.objectValues() if (parser.isAvailable() or with_unavailables) and (parser.isEnabled() or with_disabled) ]
+                for parser in parsers if (parser.isAvailable() or with_unavailables) and (parser.isEnabled() or with_disabled) ]
 
     security.declarePublic('getImportFormatExtensions')
     def getImportFormatExtensions(self, with_unavailables=False, with_disabled=False):
@@ -306,24 +326,28 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
         returns a list with the file name extensions
         of the supported import formats
         """
+        parsers = component.getAllUtilitiesRegisteredFor(IBibliographyParser)
         return [parser.getFormatExtension() \
-                for parser in self.Parsers.objectValues() if (parser.isAvailable() or with_unavailables) and (parser.isEnabled() or with_disabled)]
+                for parser in parsers if (parser.isAvailable() or with_unavailables) and (self.isParserEnabled(parser.getFormatName()) or with_disabled)]
 
     def getImportFormatDescriptions(self, with_unavailables=False, with_disabled=False):
         """
         returns a list with the description texts
         of the supported import formats
         """
+        parsers = component.getAllUtilitiesRegisteredFor(IBibliographyParser)
         return [parser.Description() \
-                for parser in self.Parsers.objectValues() if (parser.isAvailable() or with_unavailables) and (parser.isEnabled() or with_disabled) ]
+                for parser in parsers if (parser.isAvailable() or with_unavailables) and (parser.isEnabled() or with_disabled) ]
 
     security.declarePublic('getExportFormatNames')
     def getExportFormatNames(self, with_unavailables=False, with_disabled=False):
         """
         returns a list with the names of the supported export formats
         """
-        return [renderer.getFormatName() \
-                for renderer in self.Renderers.objectValues() if (renderer.isAvailable() or with_unavailables) and (renderer.isEnabled() or with_disabled) ]
+        utils = component.getAllUtilitiesRegisteredFor(IBibliographyRenderer)
+        return [ renderer.__name__ for renderer in utils
+                 if (renderer.available or with_unavailables) and
+                    (self.isRendererEnabled(renderer.__name__) or with_disabled) ]
 
     security.declarePublic('getExportFormatExtensions')
     def getExportFormatExtensions(self, with_unavailables=False, with_disabled=False):
@@ -331,16 +355,20 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
         returns a list with the file name extensions
         of the supported export formats
         """
-        return [renderer.getFormatExtension() \
-                for renderer in self.Renderers.objectValues() if (renderer.isAvailable() or with_unavailables) and (renderer.isEnabled() or with_disabled) ]
+        utils = component.getAllUtilitiesRegisteredFor(IBibliographyRenderer)
+        return [ renderer.target_format for renderer in utils
+                 if (renderer.available or with_unavailables) and
+                    (self.isRendererEnabled(renderer.__name__) or with_disabled) ]
 
     def getExportFormatDescriptions(self, with_unavailables=False, with_disabled=False):
         """
         returns a list with the description texts
         of the supported export formats
         """
-        return [renderer.Description() \
-                for renderer in self.Renderers.objectValues() if (renderer.isAvailable() or with_unavailables) and (renderer.isEnabled() or with_disabled) ]
+        utils = component.getAllUtilitiesRegisteredFor(IBibliographyRenderer)
+        return [ renderer.description for renderer in utils
+                 if (renderer.available or with_unavailables) and
+                    (self.isRendererEnabled(renderer.__name__) or with_disabled) ]
 
     security.declarePublic('getExportFormats')
     def getExportFormats(self, with_unavailables=False, with_disabled=False):
@@ -348,12 +376,16 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
         returns a list of (name, extension, description) tuples
         of the supported export formats
         """
-        export_formats = zip(self.getExportFormatNames(with_unavailables=with_unavailables, with_disabled=with_disabled),
-                             self.getExportFormatExtensions(with_unavailables=with_unavailables, with_disabled=with_disabled),
-                             self.getExportFormatDescriptions(with_unavailables=with_unavailables, with_disabled=with_disabled))
+        export_formats = zip(
+            self.getExportFormatNames(with_unavailables=with_unavailables,
+                                      with_disabled=with_disabled),
+            self.getExportFormatExtensions(with_unavailables=with_unavailables,
+                                           with_disabled=with_disabled),
+            self.getExportFormatDescriptions(with_unavailables=with_unavailables,
+                                            with_disabled=with_disabled))
         export_formats.sort()
-        return export_formats                     
-    
+        return export_formats
+
     security.declarePublic('getImportFormats')
     def getImportFormats(self, with_unavailables=False, with_disabled=False):
         """
@@ -365,18 +397,28 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
                                 self.getImportFormatDescriptions(with_unavailables=with_unavailables, with_disabled=with_disabled))
         supported_parsers.sort()
         return supported_parsers
-        
+
     security.declareProtected(View, 'render')
     def render(self, entry, format='', output_encoding=None, **kw):
         """
         renders a BibliographyEntry object in the specified format
         """
-	renderer = self.getRenderer(format=format, **kw)
+        renderer = self.getRenderer(format=format, **kw)
         if renderer:
             return renderer.render(entry, output_encoding=output_encoding, **kw)
         else:
             return None
 
+    security.declareProtected(View, 'isParserEnabled')
+    def isParserEnabled(self, name):
+        """ Check cmfbibat propertysheet """
+        return self.getSheetProperty(name, 'parser_enabled')
+
+    security.declareProtected(View, 'isRendererEnabled')
+    def isRendererEnabled(self, name):
+        """ Check cmfbibat propertysheet """
+        return self.getSheetProperty(name, 'renderer_enabled')
+
     security.declareProtected(View, 'getRenderer')
     def getRenderer(self, format, with_unavailables=False, with_disabled=False, **kw):
         """
@@ -384,28 +426,34 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
         first looks for a renderer with the 'format' name
         next looks for a renderer with the 'format' extension
         """
-        for renderer in self.Renderers.objectValues():
-            if (renderer.isAvailable() or with_unavailables) and (renderer.isEnabled() or with_disabled):
-                if format.lower() == renderer.getFormatName().lower():
-                    return renderer
-                if format.lower() == renderer.getFormatExtension().lower():
-                    return renderer
+        utils = component.getAllUtilitiesRegisteredFor(IBibliographyRenderer)
+        for renderer in utils:
+            if (renderer.available or with_unavailables) and \
+               (renderer.enabled or with_disabled):
+                if format.lower() == renderer.__name__.lower():
+                    return renderer.__of__(self)
+                if format.lower() == renderer.target_format.lower():
+                    return renderer.__of__(self)
         return None
 
+
     security.declareProtected(View, 'getEntries')
-    def getEntries(self, source, format, file_name=None):
+    def getEntries(self, source, format, file_name=None, input_encoding='utf-8'):
         """
         main routine to be called from BibliographyFolders
         returns a list with the parsed entries
         """
-        source = self.checkEncoding(source)
 
+        source = self.checkEncoding(source, input_encoding)
         format = self.checkFormat(source, format, file_name)
-
         parser = self.getParser(format)
 
         if parser:
-            return parser.getEntries(source)
+            try:
+                return parser.getEntries(source)
+            except Exception, e:
+                LOG.error('Import error while importing file (%s)' % e, exc_info=True)
+                raise RuntimeError('An error occured (%s) - please check the log file for details' % e)
         else:
             return "No parser for '%s' available." % format
 
@@ -416,13 +464,14 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
         first looks for a parser with the 'format' name
         next looks for a parser with the 'format' extension
         """
-        for parser in self.Parsers.objectValues():
+        parsers = component.getAllUtilitiesRegisteredFor(IBibliographyParser)
+        for parser in parsers:
             if (parser.isAvailable() or with_unavailables) and (parser.isEnabled() or with_disabled):
                 if format.lower() == parser.getFormatName().lower():
                     return parser
                 elif format.lower() == parser.getFormatExtension().lower():
                     return parser
-        return None ## rr: we probabliy should raise an error here 
+        return None ## rr: we probabliy should raise an error here
 
     security.declareProtected(View, 'getIdCooker')
     def getIdCooker(self, idcooker_id='etal', with_disabled=False, **kwargs):
@@ -433,14 +482,14 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
             if idcooker.isEnabled() or with_disabled:
                 if idcooker_id == idcooker.getId():
                     return idcooker
-        return None ## rr: we probabliy should raise an error here 
+        return None ## rr: we probabliy should raise an error here
 
     security.declareProtected(View, 'getDefaultIdCooker')
     def getDefaultIdCooker(self, **kwargs):
         """
         returns the site wide default id cooker (as an object)
         """
-	return self.getIdCooker(idcooker_id=self.default_idcooker_id, **kwargs)
+        return self.getIdCooker(idcooker_id=self.default_idcooker_id, **kwargs)
 
     security.declareProtected(View, 'listIdCookers')
     def listIdCookers(self, with_disabled=False):
@@ -448,44 +497,44 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
         returns a list of strings with the ids of available id cookers
         """
         idcooker_ids = [ idcooker.getId() for idcooker in self.IdCookers.objectValues() if idcooker.isEnabled() or with_disabled ]
-	idcooker_ids.sort()
-	return idcooker_ids
+        idcooker_ids.sort()
+        return idcooker_ids
 
     security.declareProtected(View, 'allowPdfUploadPortalPolicy')
     def allowPdfUploadPortalPolicy(self):
-	
-	return self.allow_pdfupload_portal_policy
-        
+
+        return self.allow_pdfupload_portal_policy
+
     security.declareProtected(View, 'allowPdfUploadForTypes')
     def allowPdfUploadForTypes(self):
-	
-	return self.allow_pdfupload_for_types
-        
+
+        return self.allow_pdfupload_for_types
+
     security.declareProtected(View, 'useParserIdsOnImport')
     def useParserIdsOnImport(self):
-	
-	return self.use_pids_on_import
+
+        return self.use_pids_on_import
 
     security.declareProtected(View, 'cookIdsOnBibRefCreation')
     def cookIdsOnBibRefCreation(self):
-    
-	return self.cook_ids_on_bibref_creation
+
+        return self.cook_ids_on_bibref_creation
 
     security.declareProtected(View, 'cookIdsAfterBibRefEdit')
     def cookIdsAfterBibRefEdit(self):
-    
-	return self.cook_ids_after_bibref_edit
+
+        return self.cook_ids_after_bibref_edit
 
     security.declareProtected(View, 'synchronizePdfFileAttributes')
     def synchronizePdfFileAttributes(self):
-    
-	return self.synchronize_pdffile_attributes
+
+        return self.synchronize_pdffile_attributes
 
     security.declareProtected(View, 'searchableBibFolders')
     def searchableBibFolders(self):
 
         return self.searchable_bibfolders
-    
+
     security.declareProtected(View, 'filterOutBibEntriesFromNonSearchableBibFolders')
     def filterOutBibEntriesFromNonSearchableBibFolders(self, search_results, object_getmethod='getObject'):
 
@@ -495,81 +544,90 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
             if object.getParentNode().portal_type in self.getBibFolderTypes() and object.getSearchable():
                 result.append(b)
         return result
-    
+
+
+    security.declareProtected(View, 'filterOutDupesByTitle')
+    def filterOutDupesByTitle(self, search_results):
+
+        result = []
+        titles_seen = []
+        for b in search_results:
+            title = b.Title
+            if not title in titles_seen:
+                result.append(b)
+                titles_seen.append(b.Title)
+        return result
+
+
     security.declareProtected(View, 'enableDuplicatesManager')
     def enableDuplicatesManager(self):
-    
-	return self.enable_duplicates_manager
+
+        return self.enable_duplicates_manager
 
     security.declareProtected(View, 'getEnableDuplicatesManager')
     def getEnableDuplicatesManager(self, instance=None):
-    
+
         if instance and getattr(instance, 'portal_type', None) and instance.portal_type in ('BibliographyFolder', 'LargeBibliographyFolder',):
             return instance.getEnableDuplicatesManager()
         else:
-    	    return self.enable_duplicates_manager
+            return self.enable_duplicates_manager
 
     security.declareProtected(View, 'inferAuthorReferencesAfterEdit')
     def inferAuthorReferencesAfterEdit(self):
-    
-	return self.infer_author_references_after_edit
+
+        return self.infer_author_references_after_edit
 
     security.declareProtected(View, 'inferAuthorReferencesAfterImport')
     def inferAuthorReferencesAfterImport(self):
-    
-	return self.infer_author_references_after_import
+
+        return self.infer_author_references_after_import
 
     security.declareProtected(View, 'authorOfImpliesOwner')
     def authorOfImpliesOwner(self):
-    
-	return self.authorof_implies_owner
+
+        return self.authorof_implies_owner
 
     security.declareProtected(View, 'authorOfImpliesCreator')
     def authorOfImpliesCreator(self):
-    
-	return self.authorof_implies_creator
-    
+
+        return self.authorof_implies_creator
+
     security.declareProtected(ManagePortal, 'transaction_savepoint')
     def transaction_savepoint(self, **kw):
-	""" needed as a wrapper for bibliography_entry_cookId script
-	"""
-	transaction.savepoint(**kw)
+        """ needed as a wrapper for bibliography_entry_cookId script
+        """
+        transaction.savepoint(**kw)
 
     security.declareProtected(View, 'cookReferenceId')
     def cookReferenceId(self, ref, idcooker_id='etal', **kwargs):
         """
         returns the cooked id for reference object
         """
-	idcooker = self.getIdCooker(idcooker_id=idcooker_id, **kwargs)
+        idcooker = self.getIdCooker(idcooker_id=idcooker_id, **kwargs)
 
-	if idcooker:
-	    return idcooker.getCookedBibRefId(ref, **kwargs)
-	    
-	return 'nobody1000'
+        if idcooker:
+            return idcooker.getCookedBibRefId(ref, **kwargs)
+
+        return 'nobody1000'
 
     def nextId(self, testid):
         letters = string.letters
-	if testid[-1] in letters:
-	    last = letters[letters.find(testid[-1])+1]
-	    return testid[:-1] + last
-	else:
-	    # FIXME BAD
-	    return testid + 'a'
-	    
-    def checkEncoding(self, source):
-        """
-        Make sure we have utf encoded text
-        """
-        mimetypesTool = getToolByName(self, 'mimetypes_registry', None)
-        encoding = mimetypesTool and mimetypesTool.guess_encoding(source) \
-                   or 'utf-8'
-        fallbackEncoding = mimetypesTool and \
-                           hasattr (mimetypesTool, 'fallbackEncoding') and \
-                           mimetypesTool.fallbackEncoding or 'latin1'
-        try:
-            source = unicode(source, encoding)
-        except UnicodeDecodeError:
-            source = unicode(source, fallbackEncoding)
+        if testid[-1] in letters:
+            last = letters[letters.find(testid[-1])+1]
+            return testid[:-1] + last
+        else:
+            # FIXME BAD
+            return testid + 'a'
+
+    def checkEncoding(self, source, input_encoding='utf-8'):
+        """ Make sure we have utf-8 encoded text """
+
+        encoding = input_encoding
+        if source.startswith(codecs.BOM_UTF8):
+            source = source[3:] # chop of BOM (might confuse checkFormat())
+            encoding = 'utf-8'
+
+        source = unicode(source, encoding)
         return source.encode('utf-8')
 
     def checkFormat(self, source, format, file_name):
@@ -578,10 +636,14 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
         if not it tries to infer the format from the 'file_name'
         raises an error if both fail
         """
+
         ok = 0
         if format:
             parser = self.getParser(format)
             ok = parser.checkFormat(source)
+        # TODO: This should be changed!  If the user requests a format, and it
+        # doesn't match, we should report that error back to the user.  Guessing
+        # results in ambiguous results.
         if not ok and file_name:
             format = self.guessFormat(file_name)
         if format:
@@ -596,24 +658,25 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
         Checks whether the file_name extension is
         among the supported ones.
 
-        returns the respective format name if found 
+        returns the respective format name if found
         returns None otherwise
         """
         extension = file_name.split('.')[-1].lower()
+        parsers = component.getAllUtilitiesRegisteredFor(IBibliographyParser)
         if extension in [ext.lower()
                          for ext in self.getImportFormatExtensions()]:
-            for parser in self.Parsers.objectValues():
+            for parser in parsers:
                 if extension == parser.getFormatExtension().lower():
                     return parser.getFormatName()
         return None
-    
+
 
     # support BibTeX style citations in text
     security.declarePublic('link_citations')
     def link_citations(self, text=""):
         """
         replace all citations with links to their references
-        
+
         the pattern is 'cite{key}' or 'cite{key1,key2}
         If 'key' matches the id of a reference the pattern
         will be replaced with inline link(s) to this reference(s).
@@ -621,7 +684,7 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
 
         Using the pattern 'bibitem{key}' you can include the
         full reference like shown in other bibliographic listings
-        (authors (year) title linked to entry, source) 
+        (authors (year) title linked to entry, source)
         """
         text = citations.sub(self._inline_links, text)
         return bibitems.sub(self._bibitem_links, text)
@@ -674,7 +737,7 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
         ids = mt.listMemberIds()
         ids.sort()
         return ids
-    
+
     security.declareProtected(View, 'getFieldIndexes')
     def getFieldIndexes(self, start_with_empty_element=False):
         """returns a list of all field index ids from the catalog"""
@@ -686,7 +749,7 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
         if start_with_empty_element:
             return [''] + field_indexes
         else:
-            return field_indexes    
+            return field_indexes
 
     security.declareProtected(View, 'getSortIndexes')
     def getSortIndexes(self, start_with_empty_element=False):
@@ -699,21 +762,7 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
         if start_with_empty_element:
             return [''] + sort_indexes
         else:
-            return sort_indexes    
-
-
-    security.declareProtected(View, 'filterOutDupesByTitle')
-    def filterOutDupesByTitle(self, search_results):
-        """ Filter out bibs with identical titles (dupe control) """
-        result = []
-        titles_seen = []
-        for b in search_results:
-            title = b.Title
-            if not title in titles_seen:
-                result.append(b)
-                titles_seen.append(b.Title)
-        return result
-
+            return sort_indexes
 
     security.declareProtected(View, 'getTextIndexes')
     def getTextIndexes(self, start_with_empty_element=False):
@@ -726,7 +775,7 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
         if start_with_empty_element:
             return [''] + text_indexes
         else:
-            return text_indexes    
+            return text_indexes
 
     security.declareProtected(View, 'getMetaDataColumns')
     def getMetaDataColumns(self, start_with_empty_element=False):
@@ -736,8 +785,8 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
         if start_with_empty_element:
             return [''] + columns
         else:
-            return columns   
-            
+            return columns
+
     security.declareProtected(ManagePortal, 'getBibliographyContentTypes')
     def getBibliographyContentTypes(self):
         """ return a sequence that contains all bibliography types"""
@@ -749,7 +798,7 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
                 if reference_type == reg_type['name']:
                     bibContentTypes.append(reg_type)
         return bibContentTypes
-													        
+
     security.declareProtected(ManagePortal, 'getBibliographyContentTypes')
     def getBibrefArchetypeName(self, classname):
         """ return the archetype name of a bibref content type class"""
@@ -758,13 +807,36 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
         if classname in bib_tool.getReferenceTypes():
             for reg_type in arche_tool.listRegisteredTypes():
                 if classname == reg_type['name']:
-		    return reg_type['klass'].archetype_name
-													        
+                    return reg_type['klass'].archetype_name
+
     security.declareProtected(View, 'usesCMFMember')
     def usesCMFMember(self):
         quickinstaller = getToolByName(self, 'portal_quickinstaller')
         return quickinstaller.isProductInstalled('CMFMember')
 
+    # The cmfbibat_properties property sheet is used to store render and
+    # parser related preferences. The 'prefix' refers to the name of
+    # the renderer or parser. Parser/Renderer names are normalized
+    # (compare with propertiestool.xml)
+
+    security.declareProtected(ManagePortal, 'updateSheetProperty')
+    def updateSheetProperty(self, prefix, property, value):
+        """ Update cmfbibat propertysheet """
+        ps = getToolByName(self, 'portal_properties').cmfbibat_properties
+        prop_name = '%s_%s' % (prefix, property)
+        prop_name = ''.join([c for c in prop_name if c.lower() in string.letters + '_'])
+        ps.manage_changeProperties(**{prop_name:value})
+
+
+    security.declareProtected(View, 'getSheetProperty')
+    def getSheetProperty(self, prefix, property):
+        """ return property from cmfbibat propertysheet """
+        ps = getToolByName(self, 'portal_properties').cmfbibat_properties
+        prop_name = '%s_%s' % (prefix, property)
+        prop_name = ''.join([c for c in prop_name if c.lower() in string.letters + '_'])
+        return ps.getProperty(prop_name)
+
+
     security.declarePrivate('getEntryDict')
     def getEntryDict(self, bibref_item, instance=None, title_link=False, title_link_only_if_owner=False, ):
         """ transform a BiblioRef Object into python dictionary
@@ -774,72 +846,73 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
                           'publication_url',
                           'abstract',
                           'note',
-		          'publisher',
-			  'editor',
+                          'publisher',
+                          'editor',
                           'editor_flag',
                           'organization',
                           'institution',
-	                  'school',
-	                  'address',
-	                  'booktitle',
+                          'school',
+                          'address',
+                          'booktitle',
                           'chapter',
-	                  'journal',
-		          'volume',
+                          'journal',
+                          'volume',
                           'edition',
                           'number',
                           'pages',
-	                  'series',
-	                  'type',
-	                  'howpublished',
-	                  'preprint_server',
-	                  'pmid',
+                          'series',
+                          'type',
+                          'howpublished',
+                          'preprint_server',
+                          'pmid',
                           'isbn',
-			  'annote',)
-	values = {}
-	tmp_title = _decode(bibref_item.Title())
-	if tmp_title and (tmp_title[-1] == '.'): tmp_title = tmp_title[:-1]
+                          'annote',)
+        values = {}
+        values['UID'] = bibref_item.UID()
+        tmp_title = _decode(bibref_item.Title())
+        if tmp_title and (tmp_title[-1] == '.'): tmp_title = tmp_title[:-1]
         values['title'] = tmp_title
         uniauthors=[]
-        for author in bibref_item.getAuthorList():
-    	    uniauthor={}
-	    for field in author.keys():
-	        uniauthor[field] = _decode(author.get(field))
-	    uniauthors.append(uniauthor)
-	values['authors'] = uniauthors
-	uniauthors=[]
-	uniauthor={}
-	for attr in ref_attributes:
+        for author in bibref_item.getAuthors():
+            uniauthor={}
+            for field in author.keys():
+                uniauthor[field] = _decode(author.get(field))
+            uniauthors.append(uniauthor)
+        values['authors'] = uniauthors
+        uniauthors=[]
+        uniauthor={}
+        for attr in ref_attributes:
             field = bibref_item.getField(attr)
-	    if field:
-    		value = getattr(bibref_item, field.accessor)()
-		if not value:
+            if field:
+                value = getattr(bibref_item, field.accessor)()
+                if not value:
                     value = field.getDefault(bibref_item)
                 try:
-		    for x in range(value.len()):
-		        value[x] = _decode(value[x])
-	            values[attr] = value
-	        except:
-		    if type(value) == type(''):
-            		values[attr] = _decode(value)
-		    else:
-		        # this is mainly for the editor flag (not a string, no unicode transformation possible
-			values[attr] = value
+                    for x in range(value.len()):
+                        value[x] = _decode(value[x])
+                    values[attr] = value
+                except:
+                    if type(value) == type(''):
+                        values[attr] = _decode(value)
+                    else:
+                        # this is mainly for the editor flag (not a string, no unicode transformation possible
+                        values[attr] = value
         values['source'] = _decode(bibref_item.Source())
         values['description'] = _decode(bibref_item.Description())
         values['meta_type'] = bibref_item.meta_type
-	values['reference_type'] = bibref_item.getReference_type() # should be same as portal_type
+        values['reference_type'] = bibref_item.getReference_type() # should be same as portal_type
         if title_link:
             mtool = getToolByName(self, 'portal_membership')
             bibref_item_roles = mtool.getAuthenticatedMember().getRolesInContext(bibref_item)
             if not title_link_only_if_owner or [ role for role in ['Owner', 'Manager'] if role in bibref_item_roles ]:
-	        values['title_link'] = bibref_item.absolute_url()
-	wf_tool = getToolByName(self, 'portal_workflow')	
-	values['review_state'] = wf_tool.getInfoFor(bibref_item, 'review_state', '')
-	if instance is not None:
-	    values['came_from'] = instance.absolute_url()
-	    values['came_from_title'] = _decode(instance.Title())
-	    values['came_from_description'] = _decode(instance.Description())
-	return values
+                values['title_link'] = bibref_item.absolute_url()
+        wf_tool = getToolByName(self, 'portal_workflow')
+        values['review_state'] = wf_tool.getInfoFor(bibref_item, 'review_state', '')
+        if instance is not None:
+            values['came_from'] = instance.absolute_url()
+            values['came_from_title'] = _decode(instance.Title())
+            values['came_from_description'] = _decode(instance.Description())
+        return values
 
     def listWfStatesForReferenceTypes(self, filter_similar=1):
 
@@ -855,10 +928,10 @@ class BibliographyTool(UniqueObject, Folder, ## ActionProviderBase,
         for statesByWorkflow in [ wtool.getWorkflowById(wf_id).states.objectValues() for wf_id in uniq_chain if wtool.getWorkflowById(wf_id) ]:
             bibref_states.extend(statesByWorkflow)
 
-	# TODO: When using CMFPlacefulWorkflow the retrieved bibref_states might be an empty list.
-	#       For this case let us for now just return the standard workflow states
-	if not bibref_states:
-	    return [ ('Private', 'private'), ('Public Draft', 'visible'), ('Pending', 'pending'), ('Published', 'published'), ]
+        # TODO: When using CMFPlacefulWorkflow the retrieved bibref_states might be an empty list.
+        #       For this case let us for now just return the standard workflow states
+        if not bibref_states:
+            return [ ('Private', 'private'), ('Public Draft', 'visible'), ('Pending', 'pending'), ('Published', 'published'), ]
 
         result = []
         dup_keys = {}
